#include "codes.h"
#include "Delay10us.h"
#include "inttypes.h"
#include "TVBGone.h"

#include "EINT_SW0.h"
#include "LED0.h"
#include "PPG1.h"


static void TVBGoneSetTimerFrequency(uint16_t Frequency)
{
	PPG1_SetFreqHz(Frequency);
}


static void TVBGoneEnableTimerOutput(void)
{
	PPG1_Enable();
}


static void TVBGoneDisableTimerOutput(void)
{
	PPG1_Disable();
	PPG1_SetValue();
}


static uint8_t TVBGoneIsButtonPressed(void)
{
	uint8_t ReturnValue = 0;

	if(FALSE == EINT_SW0_GetVal())
		ReturnValue = 1;

	return ReturnValue;
}


static void TVBGoneTurnOnLED(void)
{
	LED0_PutVal(TRUE);
}


static void TVBGoneTurnOffLED(void)
{
	LED0_PutVal(FALSE);
}


// This function quickly pulses the visible LED 
static void TVBGoneQuickflashLED(void)
{
	TVBGoneTurnOnLED();
	Cpu_Delay10US(3000);         // 30 millisec delay
	TVBGoneTurnOffLED();
}


// This function quickly pulses the visible LED x times
static void TVBGoneQuickflashLEDx(uint8_t x) 
{
   TVBGoneQuickflashLED();
   while(--x)
   {
      Cpu_Delay10US(10000);        // 100 millisec delay was 150millisec delay
      TVBGoneQuickflashLED();
   }
}


// This function transmits one Code Element of a POWER code to the IR emitter, 
//   given offTime and onTime for the codeElement
//     If offTime = 0 that signifies the last Code Element of the POWER code
//     and the delay_ten_us function will have no delay for offTime 
//     (but we'll delay for 250 milliseconds in the main function)
static void TVBGoneXmitCodeElement(uint16_t ontime, uint16_t offtime ) 
{
	TVBGoneEnableTimerOutput();

   // keep transmitting carrier for onTime
   Cpu_Delay10US(ontime);

   // turn off output to IR emitters 
   TVBGoneDisableTimerOutput();

   Cpu_Delay10US(offtime);
}


static void TVBGoneTransmitCode(uint8_t TVCode)
{
   uint16_t FrequencyValue;
   uint16_t OnTime, OffTime;
   uint8_t TimeIndex;
   uint8_t CodeCount, CodeIndex;

	/* get carrier half period and set timer half period value */
	FrequencyValue = powerCodes[TVCode]->freq_val;

	TVBGoneSetTimerFrequency(FrequencyValue);         
      
   CodeCount = 0;
   CodeIndex = 0;
   
   do
   {
      if(CodeCount & 1)
      {
         TimeIndex = (uint8_t)((powerCodes[TVCode]->codes[CodeIndex] << 2) & 0x3c);
         CodeIndex++;
      }
      else
         TimeIndex = (uint8_t)((powerCodes[TVCode]->codes[CodeIndex] >> 2) & 0x3c);
      
      /* convert TimeIndex from uint8_t index to uint16_t index */
      TimeIndex /= sizeof(uint16_t);
      
      OnTime = powerCodes[TVCode]->times[TimeIndex];
      OffTime = powerCodes[TVCode]->times[TimeIndex + 1];
      
      TVBGoneXmitCodeElement(OnTime, OffTime);         
      
      CodeCount++;   
   }
   while(0 != OffTime);
}


static void TVBGoneTransmitCodes(void)
{
   uint8_t i;
   
   for(i = 0;i < num_codes;i++)
   {
      TVBGoneTransmitCode(i);

      if(TVBGoneIsButtonPressed())
      {
         TVBGoneQuickflashLEDx(2);
         
         while(TVBGoneIsButtonPressed())
         {
         	
         }
         
         i = 0;
      }
      else
      {
         Cpu_Delay10US(12500);
         TVBGoneQuickflashLED();
      }
   }
}


void TVBGoneRun(void)
{
   if(TVBGoneIsButtonPressed())
   {
      Cpu_Delay10US(10000);
      while(TVBGoneIsButtonPressed())
      {
      	
      }
      
      TVBGoneTransmitCodes();         

      /* wait 500 ms, then quickflash LED 4 time to indicate we are done */
      Cpu_Delay10US(50000); 
      TVBGoneQuickflashLEDx(4);
   }
}


